@@ -1,11 +1,11 @@
-# $Id: POE.pm,v 1.37 1999/11/20 05:43:50 rcaputo Exp $
+# $Id: POE.pm,v 1.48 2000/01/25 19:16:07 rcaputo Exp $
 # Copyrights and documentation are after __END__.
 
 package POE;
 
 use vars qw($VERSION);
 
-$VERSION = 0.08;
+$VERSION = 0.09;
 
 use strict;
 use Carp;
@@ -216,7 +216,7 @@ The obligatory ASCII art:
 
 =head1 EXAMPLES
 
-As of this writing there are 22 sample programs.  Each illustrates and
+As of this writing there are 24 sample programs.  Each illustrates and
 tests some aspect of POE use.  They are included in the POE
 distribution archive, but they are not installed.  If POE was
 installed via the CPAN shell, then you should be able to find them in
@@ -260,6 +260,13 @@ callback states.
 
 =item *
 
+objmaps.perl
+
+This is a version of objsessions.perl that maps states to differently
+named object methods.
+
+=item *
+
 objsessions.perl
 
 This program is essentially the same as sessions.perl, but it uses
@@ -403,11 +410,25 @@ thrash.perl will show an abnormally low connections/second rate.
 
 =item *
 
+udp.perl
+
+Udp shows how to use UDP sockets with Kernel::select calls.
+
+=item *
+
 wheels.perl
 
 This program is a basic rot13 server.  It is a basic test of the whole
 premise of wheels.
 
+=item *
+
+wheels2.perl
+
+Wheels2 shows how to use separate input and output filehandles with
+wheels.  It's a simple raw tcp socket client, piping between a client
+socket and stdio (in cooked mode).
+
 =back
 
 =head2 Object Layer Examples
@@ -437,6 +458,17 @@ considered while POE was being designed.
 
 =item *
 
+poing.perl
+
+Poing is a ping program that can check multiple hosts at the same
+time.  Historical information scrolls across the screen in a "strip
+chart" fashion.  It's great for listening to the seismology of your
+local network (no, it's not deliberately a Quake reference).
+
+Poing's event-driven pinger "borrows" heavily from Net::Ping.
+
+=item *
+
 preforkedserver.perl
 
 This program demonstrates a way to write pre-forking servers with POE.
@@ -497,6 +529,8 @@ POE::Runtime
 The Object Layer is still in early design and implementation, so it's
 not documented yet.
 
+There are no automated regression tests.
+
 =head1 AUTHORS & COPYRIGHTS
 
 POE is brought to you by the following people:
