@@ -1,4 +1,4 @@
-# $Id: SysRW.pm,v 1.6 1999/01/28 03:35:57 troc Exp $
+# $Id: SysRW.pm,v 1.8 1999/11/20 05:43:51 rcaputo Exp $
 
 # Copyright 1998 Rocco Caputo <troc@netrus.net>.  All rights reserved.
 # This program is free software; you can redistribute it and/or modify
@@ -8,6 +8,7 @@ package POE::Driver::SysRW;
 
 use strict;
 use POSIX qw(EAGAIN);
+use Carp;
 
 #------------------------------------------------------------------------------
 
@@ -16,7 +17,24 @@ sub new {
   my $self = bless { 'out queue'  => [ ],
                      'bytes done' => 0,
                      'bytes left' => 0,
+                     BlockSize    => 512,
                    }, $type;
+
+  if (@_) {
+    if (@_ % 2) {
+      croak "$type requires an even number of parameters, if any";
+    }
+    my %args = @_;
+    if (exists $self->{BlockSize}) {
+      $self->{BlockSize} = delete $args{BlockSize};
+      croak "$type BlockSize must be greater than 0" if ($self->{BlockSize}<1);
+    }
+    if (keys %args) {
+      my @bad_args = sort keys %args;
+      croak "$type has unknown parameter(s): @bad_args";
+    }
+  }
+
   $self;
 }
 
@@ -38,7 +56,7 @@ sub put {
 sub get {
   my ($self, $handle) = @_;
 
-  my $result = sysread($handle, my $buffer = '', 512);
+  my $result = sysread($handle, my $buffer = '', $self->{BlockSize});
   if ($result || ($! == EAGAIN)) {
     $! = 0;
     [ $buffer ];
@@ -83,3 +101,51 @@ sub flush {
 
 ###############################################################################
 1;
+
+__END__
+
+=head1 NAME
+
+POE::Driver::SysRW - POE sysread/syswrite Abstraction
+
+=head1 SYNOPSIS
+
+  $driver = new POE::Driver::SysRW();
+  $arrayref_of_data_chunks = $driver->get($filehandle);
+  $queue_size = $driver->put($arrayref_of_data_chunks);
+  $queue_size = $driver->flush($filehandle);
+
+=head1 DESCRIPTION
+
+This driver provides an abstract interface to sysread and syswrite.
+
+=head1 PUBLIC METHODS
+
+=over 4
+
+=item *
+
+POE::Driver::SysRW::new( ... );
+
+The new() constructor accepts one optional parameter:
+
+  BlockSize => $block_size
+
+This is the maximum data size that the SysRW driver will read at once.
+If omitted, $block_size defaults to 512.
+
+=back
+
+=head1 SEE ALSO
+
+POE::Driver
+
+=head1 BUGS
+
+Oh, probably some.
+
+=head1 AUTHORS & COPYRIGHTS
+
+Please see the POE manpage.
+
+=cut
