@@ -1,4 +1,4 @@
-# $Id: Session.pm,v 1.25 1999/08/20 17:19:04 rcaputo Exp $
+# $Id: Session.pm,v 1.31 2000/01/24 19:22:06 rcaputo Exp $
 
 package POE::Session;
 
@@ -47,7 +47,11 @@ sub new {
   croak "$type requires a working Kernel"
     unless (defined $POE::Kernel::poe_kernel);
 
-  my $self = bless [ { }, { } ], $type;
+  my $self = bless [ ], $type;
+  $self->[SE_NAMESPACE] = { };
+  $self->[SE_OPTIONS  ] = { };
+  $self->[SE_KERNEL   ] = undef;
+  $self->[SE_STATES   ] = { };
 
   while (@states) {
                                         # handle arguments
@@ -70,33 +74,53 @@ sub new {
       if (ref($state) eq 'CODE') {
         croak "using a CODE reference as an event handler name is not allowed";
       }
-                                        # regular states
+
+      # regular states
       if (ref($state) eq '') {
         if (ref($handler) eq 'CODE') {
           $self->register_state($state, $handler);
           next;
         }
+
         elsif (ref($handler) eq 'ARRAY') {
           foreach my $method (@$handler) {
-            $self->register_state($method, $state);
+            $self->register_state($method, $state, $method);
           }
           next;
         }
+
+        elsif (ref($handler) eq 'HASH') {
+          while (my ($state_name, $method_name) = each %$handler) {
+            $self->register_state($state_name, $state, $method_name);
+          }
+          next;
+        }
+
         else {
           croak "using something other than a CODEREF for $state handler";
         }
       }
                                         # object states
       if (ref($handler) eq '') {
-        $self->register_state($handler, $state);
+        $self->register_state($handler, $state, $handler);
         next;
       }
-      if (ref($handler) ne 'ARRAY') {
-        croak "strange reference ($handler) used as an object session method";
+
+      if (ref($handler) eq 'ARRAY') {
+        foreach my $method (@$handler) {
+          $self->register_state($method, $state, $method);
+        }
+        next;
       }
-      foreach my $method (@$handler) {
-        $self->register_state($method, $state);
+
+      if (ref($handler) eq 'HASH') {
+        while (my ($state_name, $method_name) = each %$handler) {
+          $self->register_state($state_name, $state, $method_name);
+        }
+        next;
       }
+
+      croak "strange reference ($handler) used as an object session method";
     }
     else {
       last;
@@ -132,7 +156,11 @@ sub create {
 
   my %params = @params;
 
-  my $self = bless [ { }, { } ], $type;
+  my $self = bless [ ], $type;
+  $self->[SE_NAMESPACE] = { };
+  $self->[SE_OPTIONS  ] = { };
+  $self->[SE_KERNEL   ] = undef;
+  $self->[SE_STATES   ] = { };
 
   if (exists $params{'args'}) {
     if (ref($params{'args'}) eq 'ARRAY') {
@@ -144,36 +172,78 @@ sub create {
     delete $params{'args'};
   }
 
+  if (exists $params{options}) {
+    if (ref($params{options}) eq 'HASH') {
+      $self->[SE_OPTIONS] = $params{options};
+    }
+    else {
+      croak "options for $type constructor is expected to be a HASH reference";
+    }
+    delete $params{options};
+  }
+
   my @params_keys = keys(%params);
   foreach (@params_keys) {
-    my $state_hash = $params{$_};
+    my $states = $params{$_};
 
-    croak "$_ does not refer to a hashref"
-      unless (ref($state_hash) eq 'HASH');
+     if ($_ eq 'inline_states') {
+      croak "$_ does not refer to a hash" unless (ref($states) eq 'HASH');
 
-    if ($_ eq 'inline_states') {
-      while (my ($state, $handler) = each(%$state_hash)) {
+      while (my ($state, $handler) = each(%$states)) {
         croak "inline state '$state' needs a CODE reference"
           unless (ref($handler) eq 'CODE');
         $self->register_state($state, $handler);
       }
     }
     elsif ($_ eq 'package_states') {
-      while (my ($state, $handler) = each(%$state_hash)) {
-        croak "states for package '$state' needs an ARRAY reference"
-          unless (ref($handler) eq 'ARRAY');
-        foreach my $method (@$handler) {
-          $self->register_state($method, $state);
+      croak "$_ does not refer to an array" unless (ref($states) eq 'ARRAY');
+      croak "the array for $_ has an odd number of elements" if (@$states & 1);
+
+      while (my ($package, $handlers) = splice(@$states, 0, 2)) {
+
+        # Array of handlers is passed through as method names.
+        if (ref($handlers) eq 'ARRAY') {
+          foreach my $method (@$handlers) {
+            $self->register_state($method, $package, $method);
+          }
+        }
+
+        # Hashes of handlers are passed through as key names.
+        elsif (ref($handlers) eq 'HASH') {
+          while (my ($state, $method) = each %$handlers) {
+            $self->register_state($method, $package, $state);
+          }
+        }
+
+        else {
+          croak "states for '$package' needs to be a hash or array ref";
         }
       }
     }
     elsif ($_ eq 'object_states') {
-      while (my ($state, $handler) = each(%$state_hash)) {
-        croak "states for object '$state' need an ARRAY reference"
-          unless (ref($handler) eq 'ARRAY');
-        foreach my $method (@$handler) {
-          $self->register_state($method, $state);
+      croak "$_ does not refer to an array" unless (ref($states) eq 'ARRAY');
+      croak "the array for $_ has an odd number of elements" if (@$states & 1);
+
+      while (my ($object, $handlers) = each(%$states)) {
+
+        # Array of handlers is passed through as method names.
+        if (ref($handlers) eq 'ARRAY') {
+          foreach my $method (@$handlers) {
+            $self->register_state($method, $object, $method);
+          }
         }
+
+        # Hashes of handlers are passed through as key names.
+        elsif (ref($handlers) eq 'HASH') {
+          while (my ($state, $method) = each %$handlers) {
+            $self->register_state($method, $object, $state);
+          }
+        }
+
+        else {
+          croak "states for '$object' needs to be a hash or array ref";
+        }
+
       }
     }
     else {
@@ -221,15 +291,16 @@ sub _invoke_state {
     }
                                         # package and object
     else {
+      my ($object, $method) = @{$self->[SE_STATES]->{$state}};
       return
-        $self->[SE_STATES]->{$state}->$state(                         # object
-                                            $self,                    # session
-                                            $POE::Kernel::poe_kernel, # kernel
-                                            $self->[SE_NAMESPACE],    # heap
-                                            $state,                   # state
-                                            $source_session,          # sender
-                                            @$etc                     # args
-                                           );
+        $object->$method(                          # object
+                         $self,                    # session
+                         $POE::Kernel::poe_kernel, # kernel
+                         $self->[SE_NAMESPACE],    # heap
+                         $state,                   # state
+                         $source_session,          # sender
+                         @$etc                     # args
+                        );
     }
   }
                                         # recursive, so it does the right thing
@@ -255,9 +326,11 @@ sub _invoke_state {
 #------------------------------------------------------------------------------
 
 sub register_state {
-  my ($self, $state, $handler) = @_;
+  my ($self, $state, $handler, $method) = @_;
+  $method = $state unless defined $method;
 
   if ($handler) {
+    # Inline coderef.
     if (ref($handler) eq 'CODE') {
       carp "redefining state($state) for session($self)"
         if ( (exists $self->[SE_OPTIONS]->{'debug'}) &&
@@ -265,13 +338,15 @@ sub register_state {
            );
       $self->[SE_STATES]->{$state} = $handler;
     }
-    elsif ($handler->can($state)) {
+    # Object or package method.
+    elsif ($handler->can($method)) {
       carp "redefining state($state) for session($self)"
         if ( (exists $self->[SE_OPTIONS]->{'debug'}) &&
              (exists $self->[SE_STATES]->{$state})
            );
-      $self->[SE_STATES]->{$state} = $handler;
+      $self->[SE_STATES]->{$state} = [ $handler, $method ];
     }
+    # Something's wrong.
     else {
       if (ref($handler) eq 'CODE' &&
           exists($self->[SE_OPTIONS]->{'trace'})
@@ -280,7 +355,7 @@ sub register_state {
       }
       else {
         croak "object $handler does not have a '$state' method"
-          unless ($handler->can($state));
+          unless ($handler->can($method));
       }
     }
   }
@@ -291,6 +366,13 @@ sub register_state {
 
 #------------------------------------------------------------------------------
 
+sub ID {
+  my $self = shift;
+  $POE::Kernel::poe_kernel->ID_session_to_id($self);
+}
+
+#------------------------------------------------------------------------------
+
 sub option {
   my $self = shift;
   my %return_values;
@@ -377,14 +459,19 @@ POE::Session - POE State Machine
                         state2 => \&handler2,
                         ...
                       },
-    object_states  => { $objref1 => \@methods2,
-                        $objref2 => \@methods2,
+    object_states  => [ $objref1 => \@methods1,
+                        $objref2 => { state_name_1 => 'method_name_1',
+                                      state_name_2 => 'method_name_2',
+                                    },
                         ...
-                      },
-    package_states => { $package1 => \@function_names_1,
-                        $package2 => \@function_names_2,
+                      ],
+    package_states => [ $package1 => \@function_names_1,
+                        $package2 => { state_name_1 => 'method_name_1',
+                                       state_name_2 => 'method_name_2',
+                                     },
                         ...
-                      },
+                      ],
+    options => \%options,
   );
 
   # Set or clear some session options:
@@ -482,7 +569,9 @@ to the session's B<_start> state.
 create
 
 POE::Session::create() is a new constructor style.  It does not use
-parameter overloading and DWIM to discern different session types.
+parameter overloading and DWIM to discern different session types.  It
+also supports the ability to set options in the constructor, unlike
+POE::Session::new().
 
 Please see the SYNOPSIS for create() usage.
 
@@ -546,6 +635,13 @@ dispatched.
 
 =back
 
+=item *
+
+ID
+
+POE::Session::ID() returns this session's unique ID, as maintained by
+POE::Kernel.  It's a shortcut for $kernel->ID_session_to_id($session).
+
 =back
 
 =head1 STATE PARAMETERS
@@ -657,6 +753,18 @@ Another way to grab the arguments, no matter how many there are, is:
 
 =back
 
+=head1 CUSTOM EVENTS AND PARAMETERS
+
+Events that aren't prefixed with leading underscores may have been
+defined by the state machines themselves or by Wheel instances the
+machines are using.
+
+In almost all these cases, the event name should be mapped to a state
+in the POE::Session constructor.  Finding the event's source may be
+more difficult.  It could come from a Wheel in the same session, or
+one of the &kernel calls.  In the case of inter-session communication,
+it may even come from outside the session.
+
 =head1 PREDEFINED EVENTS AND PARAMETERS
 
 POE reserves some event names for internal and standard use.  All its
@@ -853,7 +961,7 @@ POE; POE::Kernel
 
 =head1 BUGS
 
-Oh, probably some.
+The documentation for POE::Session::create() is fairly nonexistent.
 
 =head1 AUTHORS & COPYRIGHTS
 
